/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.tali;

import com.mycompany.tali.util.Constants;
import java.util.ArrayList;
import javax.swing.JProgressBar;


/**
 *
 * @author maisner
 */
public class ImmuneSystem {
  Lymphocyte[] lymphocytes = new Lymphocyte[Constants.REQUIRED_POPULATION];
  
  /**
   * Naplní populaci lymfocyty.
   */
  public void generateLymphocytes(){
    
    Lymphocyte newLymphocyte;
    
    // Testovací cyklus. Ten zaručí, že nebudou duplicity.
    for(int i = 0; i < Constants.REQUIRED_POPULATION; i++){
      
      newLymphocyte = new Lymphocyte();
      
      // Otestuj nově vytvořený lymfocyt. Jestli reaguje
      // na vlastní lymfocyt, vygeneruj nový a zresetuj počítadlo.
      for(int j = 0; j < i; j++) {
        if(newLymphocyte.isMatchedWith(lymphocytes[j])){
          newLymphocyte = new Lymphocyte();
          // Jelikož dojde k iteraci, pak je potřeba dát -1 (-1+1 = 0)
          j = -1;
        }
      }
      
      lymphocytes[i] = newLymphocyte;
    }

    
    
  }
  
  /**
   * Natrénuje lymfocyty. U každého lymfocytu určí schopnost detekovat
   * správně spam.
   * @param messages 
   * @param spam 
   */
  public void trainLymphocytes(String[] messages, boolean spam, JProgressBar pb){
    double spamAdd = 0;

    pb.setMaximum(messages.length);
    int i = 0;
    for(String message : messages){
      // Pokud je předklasifikovaná zpráva spam, zjistíme 
      // s jakou pravděpodobností tomu tak je (definovano zdrojem).
      if(spam){
        spamAdd = Constants.LIKEHOOD_OF_CLASSIFIED_SPAM;
      }
      
      // Pro každý lymfocyt, který se naváže na tuto zprávu zvýšíme počet zpráv
      // na které reagoval o jednu. A počet zaznamenaného spamu o pravděpodobnost,
      // že zpráva je spam když je klasifikována jako spam (může být 0)
      for(Lymphocyte lymphocyte : lymphocytes){
        if(lymphocyte.isMatchedWith(message)){
          lymphocyte.setMessagesMatched(lymphocyte.getMessagesMatched() + 1);
          lymphocyte.setSpamMatched(lymphocyte.getSpamMatched() + spamAdd);
        }
      }
      
      pb.setValue(++i);
    }
  }
  
  /**
   * Pouižiji natrénované lymfocyty na danou zprávu.
   * @param message 
   * @return  
   */
  public boolean applyLymphocytes(String message){
    // Hodnoty pro označení kolikrát byla zpráva označena jako spam,
    // Bude se zvyšovat o počet detekcí spamu a  zpráv daného lymfocytu,
    // což zajistí, že nejvíce budou ovlivňovat tyto hodnoty lymfocyty,
    // které správně detekovaly spam.
    double totalSpamMatched = 0;
    double totalMessagesMatched = 0;
    
    // Nevím, kolik lymfocytů bude na zprávu reagovat,
    // proto nelze použít pole.
    ArrayList<Lymphocyte> matchedLymphocytes = new ArrayList<>();
    
    for(Lymphocyte lymphocyte : lymphocytes){
      if(lymphocyte.isMatchedWith(message)){
        // Připočtení o hodnoty, kolikrát se již správně trefil.
        totalSpamMatched += lymphocyte.getSpamMatched();
        totalMessagesMatched += lymphocyte.getMessagesMatched();
        
        lymphocyte.setMessagesMatched(lymphocyte.getMessagesMatched() + 1);
        matchedLymphocytes.add(lymphocyte);
      }
    }
    
    double score;
    if (totalMessagesMatched == 0) {
      score = 0;
    } else {
      score = totalSpamMatched / totalMessagesMatched;
    }
    
    // Každý lymfocyt, který se trefil se při celkovém označení zprávy jako spam, bude
    // označen o toto score spolehlivějším při detekci dalšího spamu.
    if(score >= Constants.TRESHOLD_CLASSIFICATION){
      double spamIncrement = score * Constants.CONFIDENCE;
      for(Lymphocyte lymphocyte : matchedLymphocytes){
        lymphocyte.setMessagesMatched(lymphocyte.getMessagesMatched() + spamIncrement);
      }
      return true;
    }
    else{
      return false;
    }
  }
  
  /**
   * Metoda modelující odebírání jednotlivců z populace a nahrazování
   * novými. Zde je pro úplnost, program by tuto metodu mohl volat například
   * pokud by bežel deset dní bez zastavení (v případě intervalu obnovy 10 dní).
   */
  public void repopulateLymphocytes(){
    double spamMatched;
    double messagesMatched;
    
    for(int i = 0; i < lymphocytes.length; i++){
      spamMatched = lymphocytes[i].getSpamMatched();
      messagesMatched = lymphocytes[i].getMessagesMatched();

      // Pro zachování poměru mezi spam a odhalenými zprávami
      spamMatched = spamMatched/messagesMatched * (messagesMatched - Constants.DECREMENT);
      messagesMatched = messagesMatched - Constants.DECREMENT;
      
      // Pokud je pod práh, pak odeber a nahraď novým.
      if(messagesMatched < Constants.MESSAGES_TRESHOLD){
        lymphocytes[i] = null;
        lymphocytes[i] = new Lymphocyte();
      }
      else{
        lymphocytes[i].setMessagesMatched(messagesMatched);
        lymphocytes[i].setSpamMatched(spamMatched);
      }

    }

  }
  

  
}
